--***************************************************************
--* File         : TB_WBIPRT.vhd                                *
--* Created      : time-stamp: <08/05/12 01:10:36 kawasan>      *
--* Modified     : Time-stamp: <08/05/12 01:29:22 kawasan>      *
--*								*
--* Test-Bench                                                  *
--*								*
--* Copyright 2008 Osamu Kawashima. all rights reserved		*
--***************************************************************

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;

use std.textio.all;
use ieee.std_logic_textio.all;

entity TB_WBIPRT is end;

architecture behavior of TB_WBIPRT is
    
	component WBIPRT
	generic (
		DATA_SIZE       : integer := 32
	);
	port (
		-- WISHBONE SLAVE interface:
		WB_CLK_I        : in    std_logic;
		WB_RST_I        : in    std_logic;
		WB_DAT_I        : in    std_logic_vector(31 downto 0);
		WB_WE_I         : in    std_logic;
		WB_STB_I        : in    std_logic;
		WB_ACK_O        : out   std_logic;
		WB_DAT_O        : out   std_logic_vector(31 downto 0);

		-- Input port (non-WISHBONE signals):
		PRT_I           : in    std_logic_vector(DATA_SIZE-1 downto 0)
	);
	end component;

	constant ZERO32         : std_logic_vector(31 downto 0) := (others=>'0');
	constant ONE32          : std_logic_vector(31 downto 0) := (others=>'1');

	constant DATA_SIZE      : integer := 32;
	--constant DATA_SIZE      : integer := 8;

	constant WT1            : Time := 10 ns;
	constant WT_DELAY       : Time := 1 ns;
	constant WT_IDLE        : Time := 100 ns;
	constant CLK_CYCLE      : Time := 21 ns;	-- about 48MHz

	signal WB_CLK_I         : std_logic := '0';
	signal WB_RST_I         : std_logic := '1';
	signal WB_ADR_I         : std_logic_vector(31 downto 0) := (others=>'0');
	signal WB_DAT_I         : std_logic_vector(31 downto 0) := (others=>'0');
	signal WB_WE_I          : std_logic := '0';
	signal WB_STB_I         : std_logic := '0';
	signal WB_ACK_O         : std_logic := '0';
	signal WB_DAT_O         : std_logic_vector(31 downto 0) := (others=>'0');

	signal PRT_I            : std_logic_vector(DATA_SIZE-1 downto 0) := (others=>'0');

	signal test_code        : std_logic_vector(31 downto 0) := (others=>'0');
	signal rd_data          : std_logic_vector(31 downto 0) := (others=>'0');
	signal rd_dat           : std_logic_vector(31 downto 0) := (others=>'0');

	-- WB Read (32bit)
	procedure rd_uw(
		ADD_I		: in    std_logic_vector(31 downto 0);
		DAT_I		: in    std_logic_vector(31 downto 0);
	signal	ADD_O		: out   std_logic_vector(31 downto 0);
	signal	DAT_O		: out   std_logic_vector(31 downto 0);
	signal	STB		: out   std_logic;
	signal	WE		: out   std_logic
	) is
	begin
		wait until WB_CLK_I'event and WB_CLK_I='1';
		wait for WT_DELAY;
		ADD_O	<= ADD_I;
		STB	<= '1';
		WE	<= '0';
		wait until WB_CLK_I'event and WB_CLK_I='1';
		wait for WT_DELAY;
		DAT_O	<= DAT_I;
		wait for WT_DELAY;
		ADD_O	<= (others=>'0');
		STB	<= '0';
		WE	<= '0';
	end rd_uw;

	-- WB Write (32bit)
	procedure wr_uw(
		ADD_I		: in    std_logic_vector(31 downto 0);
		DAT_I		: in    std_logic_vector(31 downto 0);
	signal	ADD_O		: out   std_logic_vector(31 downto 0);
	signal	DAT_O		: out   std_logic_vector(31 downto 0);
	signal	STB		: out   std_logic;
	signal	WE		: out   std_logic
	) is
	begin
		wait until WB_CLK_I'event and WB_CLK_I='1';
		wait for WT_DELAY;
		ADD_O	<= ADD_I;
		DAT_O	<= DAT_I;
		STB	<= '1';
		WE	<= '1';
		wait until WB_CLK_I'event and WB_CLK_I='1';
		wait for WT_DELAY;
		ADD_O	<= (others=>'0');
		DAT_O	<= (others=>'0');
		STB	<= '0';
		WE	<= '0';
	end wr_uw;

begin
    
	uWBIPRT : WBIPRT
	generic map (
		DATA_SIZE       => DATA_SIZE
	)
	port map (
		-- WISHBONE SLAVE interface:
		WB_CLK_I        => WB_CLK_I,
		WB_RST_I        => WB_RST_I,
		WB_DAT_I        => WB_DAT_I,
		WB_WE_I         => WB_WE_I,
		WB_STB_I        => WB_STB_I,
		WB_ACK_O        => WB_ACK_O,
		WB_DAT_O        => WB_DAT_O,

		-- Input port (non-WISHBONE signals):
		PRT_I           => PRT_I
	);

	process(WB_RST_I,WB_CLK_I)
	begin
		if WB_RST_I='1' then
			rd_dat <= (others=>'0');
		elsif WB_CLK_I'event and WB_CLK_I='1' then
			if WB_STB_I='1' and WB_WE_I='0' then
				rd_dat <= WB_DAT_O;
			else
				rd_dat <= rd_dat;
			end if;
		end if;
	end process;

	process begin
		WB_CLK_I <= '0';
		wait for CLK_CYCLE/2;
		WB_CLK_I <= '1';
		wait for CLK_CYCLE/2;
	end process;

	process
		variable ln         : line;
	begin
		-- Reset
		wait for WT_IDLE+WT_DELAY;
		WB_RST_I <= '0';
		wait for WT_IDLE;

		-- Init
		wr_uw(x"00000000", x"55AA55AA", WB_ADR_I, WB_DAT_I, WB_STB_I, WB_WE_I);	-- dummy
		rd_uw(x"00000000", WB_DAT_O, WB_ADR_I, rd_data, WB_STB_I, WB_WE_I);

		-- Test-1
		wait for WT_IDLE;
		test_code <= x"12345678";
		rd_uw(x"00000000", WB_DAT_O, WB_ADR_I, rd_data, WB_STB_I, WB_WE_I);

		-- Test-2
		wait for WT_IDLE;
		test_code <= x"87654321";
		rd_uw(x"00000000", WB_DAT_O, WB_ADR_I, rd_data, WB_STB_I, WB_WE_I);

		-- Test-3
		wait for WT_IDLE;
		test_code <= x"55AA55AA";
		rd_uw(x"00000000", WB_DAT_O, WB_ADR_I, rd_data, WB_STB_I, WB_WE_I);

		-- End
		write(ln, string'("--- End of Test-Bench. ---"));
		writeline(output, ln);
		wait;
	end process;

	PRT_I <= test_code(PRT_I'range);

end behavior;
