--***************************************************************
--* File         : WBIPRT.vhd                                   *
--* Created      : 2007.11.19 kawasan                           *
--* Modified     : Time-stamp: <08/05/12 01:30:35 kawasan>      *
--*                                                             *
--* WISHBONE Parallel-Input-Port Controller                     *
--*                                                             *
--* Copyright 2007-2008 Osamu Kawashima. all rights reserved    *
--***************************************************************

library ieee;
use ieee.std_logic_1164.all;

entity WBIPRT is
	generic (
		DATA_SIZE       : integer := 32
	);
	port (
		-- WISHBONE SLAVE interface:
		WB_CLK_I        : in    std_logic;
		WB_RST_I        : in    std_logic;
		WB_DAT_I        : in    std_logic_vector(31 downto 0);
		WB_WE_I         : in    std_logic;
		WB_STB_I        : in    std_logic;
		WB_ACK_O        : out   std_logic;
		WB_DAT_O        : out   std_logic_vector(31 downto 0);

		-- Input port (non-WISHBONE signals):
		PRT_I           : in    std_logic_vector(DATA_SIZE-1 downto 0)
	);
end WBIPRT;

architecture RTL of WBIPRT is

	constant ZERO32         : std_logic_vector(31 downto 0) := (others=>'0');

begin

	WB_ACK_O <= WB_STB_I;

	uPRT_32BIT:
	if DATA_SIZE=32 generate
		WB_DAT_O <= PRT_I;
	end generate;

	uPRT_OTHER:
	if DATA_SIZE/=32 generate
		WB_DAT_O <= ZERO32(31 downto PRT_I'left+1) & PRT_I;
	end generate;

end RTL;
